@@ -1,6 +1,6 @@
 #############################################################################
 ##
-## Copyright (C) 2021 Riverbank Computing Limited.
+## Copyright (C) 2023 Riverbank Computing Limited.
 ## Copyright (C) 2006 Thorsten Marek.
 ## All right reserved.
 ##
@@ -184,24 +184,31 @@ class ProxyBase(metaclass=ProxyMetaclass):
 class ProxyClass(ProxyBase):
     flags = 0
 
-    def __init__(self, objectname, is_attribute, args=(), noInstantiation=False):
-        if objectname:
+    def __init__(self, object_name, ctor_args=None, ctor_kwargs=None,
+            is_attribute=False, no_instantiation=True):
+        if object_name:
             if is_attribute:
-                objectname = "self." + objectname
+                object_name = 'self.' + object_name
 
-            self._uic_name = objectname
+            self._uic_name = object_name
         else:
             self._uic_name = "Unnamed"
 
-        if not noInstantiation:
-            funcall = "%s(%s)" % \
+        if not no_instantiation:
+            args = [] if ctor_args is None else list(map(str, ctor_args))
+
+            if ctor_kwargs is not None:
+                for k, v in ctor_kwargs.items():
+                    args.append(k + '=' + str(v))
+
+            fun_call = '%s(%s)' % \
                     (moduleMember(self.module, self.__class__.__name__),
-                    ", ".join(map(str, args)))
+                    ', '.join(args))
 
-            if objectname:
-                funcall = "%s = %s" % (objectname, funcall)
+            if object_name:
+                fun_call = '%s = %s' % (object_name, fun_call)
 
-            write_code(funcall)
+            write_code(fun_call)
     
     def __str__(self):
         return self._uic_name
@@ -386,13 +393,11 @@ class QtWidgets(ProxyNamespace):
             return Literal("%s.indexOf(%s)" % (self, page))
 
         def layout(self):
-            return QtWidgets.QLayout("%s.layout()" % self,
-                    False, (), noInstantiation=True)
+            return QtWidgets.QLayout('%s.layout()' % self)
 
     class QAbstractScrollArea(QFrame):
         def viewport(self):
-            return QtWidgets.QWidget("%s.viewport()" % self, False, (),
-                    noInstantiation=True)
+            return QtWidgets.QWidget('%s.viewport()' % self)
 
     class QGraphicsView(QAbstractScrollArea): pass
     class QMdiArea(QAbstractScrollArea): pass
@@ -409,17 +414,14 @@ class QtWidgets(ProxyNamespace):
 
     class QTableView(QAbstractItemView):
         def horizontalHeader(self):
-            return QtWidgets.QHeaderView("%s.horizontalHeader()" % self,
-                    False, (), noInstantiation=True)
+            return QtWidgets.QHeaderView('%s.horizontalHeader()' % self)
 
         def verticalHeader(self):
-            return QtWidgets.QHeaderView("%s.verticalHeader()" % self,
-                    False, (), noInstantiation=True)
+            return QtWidgets.QHeaderView('%s.verticalHeader()' % self)
 
     class QTreeView(QAbstractItemView):
         def header(self):
-            return QtWidgets.QHeaderView("%s.header()" % self,
-                    False, (), noInstantiation=True)
+            return QtWidgets.QHeaderView('%s.header()' % self)
 
     class QUndoView(QListView): pass
 
@@ -441,20 +443,18 @@ class QtWidgets(ProxyNamespace):
 
     class QTreeWidgetItem(ProxyClass):
         def child(self, index):
-            return QtWidgets.QTreeWidgetItem("%s.child(%i)" % (self, index),
-                    False, (), noInstantiation=True)
+            return QtWidgets.QTreeWidgetItem('%s.child(%i)' % (self, index))
 
     class QTreeWidget(QTreeView):
         setSortingEnabled = i18n_void_func("setSortingEnabled")
         isSortingEnabled = i18n_func("isSortingEnabled")
 
         def headerItem(self):
-            return QtWidgets.QWidget("%s.headerItem()" % self, False, (),
-                    noInstantiation=True)
+            return QtWidgets.QWidget('%s.headerItem()' % self)
 
         def topLevelItem(self, index):
-            return QtWidgets.QTreeWidgetItem("%s.topLevelItem(%i)" % (self, index),
-                    False, (), noInstantiation=True)
+            return QtWidgets.QTreeWidgetItem(
+                    '%s.topLevelItem(%i)' % (self, index))
 
     class QAbstractButton(QWidget): pass
     class QCheckBox(QAbstractButton): pass
